/*----------------------------------------------------------------------
  File    : distmat.h
  Contents: distance matrix management
  Author  : Christian Borgelt
  History : 2007.01.04 file created
            2007.01.10 first version completed
            2007.01.19 collection of largest distances added
            2007.01.21 function dm_nbgraph() added
            2013.08.20 adapted to definition of type DIMID in matrix.h
----------------------------------------------------------------------*/
#ifndef __DISTMAT__
#define __DISTMAT__
#include "matrix.h"
#include "table.h"

#if ATTID_MAX != DIMID_MAX
#error "distmat requires ATTID_MAX == DIMID_MAX"
#endif
#if VALID_MAX != DIMID_MAX
#error "distmat requires VALID_MAX == DIMID_MAX"
#endif
#if TPLID_MAX != DIMID_MAX
#error "distmat requires TPLID_MAX == DIMID_MAX"
#endif

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct {                /* --- distance matrix --- */
  DIMID  rowcnt;                /* number of matrix rows */
  DIMID  objcnt;                /* number of considered objects */
  double min;                   /* minimum distance */
  double max;                   /* maximum distance */
  DIMID  **ids;                 /* object identifiers */
  double *dists[1];             /* matrix of distances */
} DISTMAT;                      /* (distance matrix) */

/*----------------------------------------------------------------------
  Constants
----------------------------------------------------------------------*/
/* --- parameters for dv_tform, dm_tform --- */
extern const double dm_sqrt  [2];
extern const double dm_square[2];

/*----------------------------------------------------------------------
  Distance Vector Functions
----------------------------------------------------------------------*/
extern void     dv_frommat (double *dists, DIMID rowcnt,
                            DIMID  *ids, DIMID objcnt,
                            MATRIX *mat, double *vec);
extern void     dv_fromtab (double *dists, DIMID rowcnt,
                            DIMID  *ids, DIMID objcnt,
                            TABLE  *tab, TUPLE *tpl);
extern void     dv_tform   (double *dists, DIMID cnt,
                            const double *pars);

/*----------------------------------------------------------------------
  Distance Matrix Functions
----------------------------------------------------------------------*/
extern DISTMAT* dm_create  (DIMID rowcnt, DIMID objcnt);
extern void     dm_delete  (DISTMAT *dm);
extern DIMID    dm_rowcnt  (DISTMAT *dm);
extern DIMID    dm_objcnt  (DISTMAT *dm);

extern double   dm_getdist (DISTMAT *dm, DIMID rowid, DIMID idx,
                            DIMID *id);
extern void     dm_setdist (DISTMAT *dm, DIMID rowid, DIMID idx,
                            DIMID id, double dist);
extern void     dm_tform   (DISTMAT *dm, const double *pars);

extern void     dm_fromtab (DISTMAT *dm, TABLE *tab);
extern void     dm_frommat (DISTMAT *dm, MATRIX *mat);
extern double   dm_fromnbm (DISTMAT *dm, DISTMAT *nbm, double max);

#ifndef NDEBUG
extern void     dm_show    (DISTMAT *dm);
#endif
/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
#define dm_rowcnt(m)       ((m)->rowcnt)
#define dm_objcnt(m)       ((m)->objcnt)

#endif
